@@ -160,6 +160,10 @@ class Wizard(BaseFrontend):
         # For above settings to apply automatically we need to indicate that we
         # are inside a full KDE session.
         os.environ["KDE_FULL_SESSION"] = "TRUE"
+        # We also need to indicate version as otherwise KDElibs3 compatibility
+        # might kick in such as in QIconLoader.cpp:QString fallbackTheme.
+        # http://goo.gl/6LkM7X
+        os.environ["KDE_SESSION_VERSION"] = "4"
 
         self.app = QtGui.QApplication([])
         # The "hicolor" icon theme gets picked when Ubiquity is running as a
@@ -647,7 +651,7 @@ class Wizard(BaseFrontend):
             if not_current and p == current_page:
                 continue
             prefix = p.ui.get('plugin_prefix')
-            for w in p.widgets:
+            for w in p.widgets + p.optional_widgets:
                 for c in self.all_children(w):
                     widgets.append((c, prefix))
 
@@ -960,7 +964,13 @@ class Wizard(BaseFrontend):
             # ShutdownConfirmNo, ShutdownTypeReboot, ShutdownModeForceNow
             ksmserver.logout(0, 1, 2)
         else:
-            misc.execute_root('reboot')
+            # don't let reboot race with the shutdown of X in ubiquity-dm;
+            # reboot might be too fast and X will stay around forever instead
+            # of moving to plymouth
+            misc.execute_root(
+                'sh', '-c',
+                "if ! service display-manager status; then killall Xorg; "
+                "while pidof X; do sleep 0.5; done; fi; reboot")
 
     def do_shutdown(self):
         """Callback for main program to actually shutdown the machine."""
@@ -975,7 +985,13 @@ class Wizard(BaseFrontend):
             # ShutdownConfirmNo, ShutdownTypeReboot, ShutdownModeForceNow
             ksmserver.logout(0, 2, 2)
         else:
-            misc.execute_root('poweroff')
+            # don't let poweroff race with the shutdown of X in ubiquity-dm;
+            # poweroff might be too fast and X will stay around forever instead
+            # of moving to plymouth
+            misc.execute_root(
+                'sh', '-c',
+                "if ! service display-manager status; then killall Xorg; "
+                "while pidof X; do sleep 0.5; done; fi; poweroff")
 
     def quit(self):
         """Quit installer cleanly."""
