@@ -286,19 +286,32 @@ class PageKde(plugin.PluginUI):
     @plugin.only_this_page
     def on_keyboard_layout_selected(self, *args):
         layout = self.get_keyboard()
-        lang = 'C'
-        if layout is not None:
-            # skip updating keyboard if not using display
-            if self.keyboardDisplay:
-                ly = keyboard_names.layout_id(lang, misc.utf8(layout))
-                self.keyboardDisplay.setLayout(ly)
+        lang = self.controller.dbfilter.get_locale()
+        try:
+            if layout is not None:
+                # skip updating keyboard if not using display
+                if self.keyboardDisplay:
+                    ly = keyboard_names.layout_id(lang, misc.utf8(layout))
+                    self.keyboardDisplay.setLayout(ly)
 
-                # no variants, force update by setting none
-                # if not keyboard_names.has_variants(l, ly):
-                #    self.keyboardDisplay.setVariant(None)
+                    # no variants, force update by setting none
+                    # if not keyboard_names.has_variants(l, ly):
+                    #    self.keyboardDisplay.setVariant(None)
 
-            self.current_layout = layout
-            self.controller.dbfilter.change_layout(layout)
+                self.current_layout = layout
+                self.controller.dbfilter.change_layout(layout)
+        except KeyError:
+            from PyQt4.QtGui import QMessageBox
+            QMessageBox.warning(self.page, 'Installer bug',
+                                'Could not find text for ' + str(layout) +
+                                ' see <a href="https://bugs.launchpad.net/' +
+                                'ubuntu/+source/ubiquity/+bug/1182784">bug ' +
+                                '1182784</a>.<br /><br /> ' +
+                                'Please select another and reconfigure ' +
+                                'after install.')
+            self.debug('Could not find text for ' + str(layout) +
+                       ' see https://bugs.launchpad.net/' +
+                       'ubuntu/+source/ubiquity/+bug/1182784')
 
     @plugin.only_this_page
     def on_keyboard_variant_selected(self, *args):
@@ -307,7 +320,7 @@ class PageKde(plugin.PluginUI):
 
         if self.keyboardDisplay:
             var = None
-            lang = 'C'
+            lang = self.controller.dbfilter.get_locale()
             ly = keyboard_names.layout_id(lang, layout)
             if variant:
                 try:
@@ -337,7 +350,7 @@ class PageKde(plugin.PluginUI):
             self.page.keyboard_layout_combobox.setCurrentIndex(index)
 
         if self.keyboardDisplay:
-            lang = 'C'
+            lang = self.controller.dbfilter.get_locale()
             ly = keyboard_names.layout_id(lang, misc.utf8(layout))
             self.keyboardDisplay.setLayout(ly)
 
@@ -361,7 +374,7 @@ class PageKde(plugin.PluginUI):
             self.page.keyboard_variant_combobox.setCurrentIndex(index)
 
         if self.keyboardDisplay:
-            lang = 'C'
+            lang = self.controller.dbfilter.get_locale()
             layout = keyboard_names.layout_id(lang, self.get_keyboard())
             if variant:
                 try:
