@@ -17,33 +17,38 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
+from __future__ import print_function
+
 import os
-import time
 import re
+import time
+from urllib.parse import quote
 
 import debconf
 import icu
 
-from ubiquity import plugin
-from ubiquity import i18n
-from ubiquity import misc
+from ubiquity import i18n, misc, plugin
 import ubiquity.tz
 
+
 NAME = 'timezone'
-#after partman for default install, but language for oem install
+# after partman for default install, but language for oem install
 AFTER = ['partman', 'language']
 WEIGHT = 10
 
 _geoname_url = 'http://geoname-lookup.ubuntu.com/?query=%s&release=%s'
 
+
 class PageGtk(plugin.PluginUI):
     plugin_title = 'ubiquity/text/timezone_heading_label'
+
     def __init__(self, controller, *args, **kwargs):
         self.controller = controller
         from gi.repository import Gtk
         builder = Gtk.Builder()
         self.controller.add_builder(builder)
-        builder.add_from_file(os.path.join(os.environ['UBIQUITY_GLADE'], 'stepLocation.ui'))
+        builder.add_from_file(os.path.join(
+            os.environ['UBIQUITY_GLADE'], 'stepLocation.ui'))
         builder.connect_signals(self)
         self.page = builder.get_object('stepLocation')
         self.city_entry = builder.get_object('timezone_city_entry')
@@ -61,12 +66,12 @@ class PageGtk(plugin.PluginUI):
         self.online = state
 
     def plugin_translate(self, lang):
-        #c = self.controller
-        #if c.get_string('ubiquity/imported/time-format', lang) == '12-hour':
+        # c = self.controller
+        # if c.get_string('ubiquity/imported/time-format', lang) == '12-hour':
         #    fmt = c.get_string('ubiquity/imported/12-hour', lang)
-        #else:
+        # else:
         #    fmt = c.get_string('ubiquity/imported/24-hour', lang)
-        #self.tzmap.set_time_format(fmt)
+        # self.tzmap.set_time_format(fmt)
         inactive = self.controller.get_string(
             'timezone_city_entry_inactive_label', lang)
         self.city_entry.set_placeholder_text(inactive)
@@ -89,10 +94,8 @@ class PageGtk(plugin.PluginUI):
             self.timezone = city
             self.controller.allow_go_forward(True)
 
-
     def changed(self, entry):
-        import urllib
-        from gi.repository import Gtk, GObject, Soup
+        from gi.repository import Gtk, GObject, GLib, Soup
 
         text = misc.utf8(self.city_entry.get_text())
         if not text:
@@ -109,16 +112,15 @@ class PageGtk(plugin.PluginUI):
 
             if self.geoname_session is None:
                 self.geoname_session = Soup.SessionAsync()
-            url = _geoname_url % (urllib.quote(text.encode('UTF-8')),
-                                  misc.get_release().version)
+            url = _geoname_url % (quote(text), misc.get_release().version)
             message = Soup.Message.new('GET', url)
             message.request_headers.append('User-agent', 'Ubiquity/1.0')
             self.geoname_session.abort()
             if self.geoname_timeout_id is not None:
-                GObject.source_remove(self.geoname_timeout_id)
+                GLib.source_remove(self.geoname_timeout_id)
             self.geoname_timeout_id = \
-                GObject.timeout_add_seconds(2, self.geoname_timeout,
-                                            (text, model))
+                GLib.timeout_add_seconds(2, self.geoname_timeout,
+                                         (text, model))
             self.geoname_session.queue_message(message, self.geoname_cb,
                                                (text, model))
 
@@ -128,11 +130,12 @@ class PageGtk(plugin.PluginUI):
             return
 
         # TODO benchmark this
-        results = [(name, self.tzdb.get_loc(city))
-                    for (name, city) in
-                        [(x[0], x[1]) for x in self.zones
-                            if x[0].lower().split('(', 1)[-1] \
-                                            .startswith(text.lower())]]
+        results = [
+            (name, self.tzdb.get_loc(city))
+            for name, city in [
+                (x[0], x[1])
+                for x in self.zones
+                if x[0].lower().split('(', 1)[-1].startswith(text.lower())]]
         for result in results:
             # We use name rather than loc.human_zone for i18n.
             # TODO this looks pretty awful for US results:
@@ -152,12 +155,12 @@ class PageGtk(plugin.PluginUI):
     def geoname_cb(self, session, message, user_data):
         import syslog
         import json
-        from gi.repository import GObject, Soup
+        from gi.repository import GLib, Soup
 
         text, model = user_data
 
         if self.geoname_timeout_id is not None:
-            GObject.source_remove(self.geoname_timeout_id)
+            GLib.source_remove(self.geoname_timeout_id)
             self.geoname_timeout_id = None
         self.geoname_add_tzdb(text, model)
 
@@ -166,29 +169,28 @@ class PageGtk(plugin.PluginUI):
             pass
         elif message.status_code != Soup.KnownStatusCode.OK:
             # Log but otherwise ignore failures.
-            syslog.syslog(('Geoname lookup for "%s" failed: %d %s' %
-                           (text, message.status_code,
-                            message.reason_phrase)).encode('UTF-8'))
+            syslog.syslog(
+                'Geoname lookup for "%s" failed: %d %s' %
+                (text, message.status_code, message.reason_phrase))
         else:
             try:
                 for result in json.loads(message.response_body.data):
-                    model.append([result['name'],
-                                result['admin1'],
-                                result['country'],
-                                result['latitude'],
-                                result['longitude']])
+                    model.append([
+                        result['name'], result['admin1'], result['country'],
+                        result['latitude'], result['longitude']])
 
                 # Only cache positive results.
                 self.geoname_cache[text] = model
 
             except ValueError:
-                syslog.syslog('Server return does not appear to be valid JSON.')
+                syslog.syslog(
+                    'Server return does not appear to be valid JSON.')
 
         self.city_entry.get_completion().set_model(model)
 
     def setup_page(self):
-        # TODO Put a frame around the completion to add contrast (LP: #605908)
-        from gi.repository import Gtk, GObject
+        # TODO Put a frame around the completion to add contrast (LP: # 605908)
+        from gi.repository import Gtk, GLib
         from gi.repository import TimezoneMap
         self.tzdb = ubiquity.tz.Database()
         self.tzmap = TimezoneMap.TimezoneMap()
@@ -200,10 +202,11 @@ class PageGtk(plugin.PluginUI):
             return m[i][0] is None
 
         self.timeout_id = 0
+
         def queue_entry_changed(entry):
             if self.timeout_id:
-                GObject.source_remove(self.timeout_id)
-            self.timeout_id = GObject.timeout_add(300, self.changed, entry)
+                GLib.source_remove(self.timeout_id)
+            self.timeout_id = GLib.timeout_add(300, self.changed, entry)
 
         self.city_entry.connect('changed', queue_entry_changed)
         completion = Gtk.EntryCompletion()
@@ -240,6 +243,7 @@ class PageGtk(plugin.PluginUI):
         completion.set_match_func(match_func, None)
         completion.set_cell_data_func(cell, data_func, None)
 
+
 class PageKde(plugin.PluginUI):
     plugin_breadcrumb = 'ubiquity/text/breadcrumb_timezone'
 
@@ -254,9 +258,11 @@ class PageKde(plugin.PluginUI):
             self.page.map_frame.layout().addWidget(self.tzmap)
 
             self.tzmap.zoneChanged.connect(self.mapZoneChanged)
-            self.page.timezone_zone_combo.currentIndexChanged[int].connect(self.regionChanged)
-            self.page.timezone_city_combo.currentIndexChanged[int].connect(self.cityChanged)
-        except Exception, e:
+            self.page.timezone_zone_combo.currentIndexChanged[int].connect(
+                self.regionChanged)
+            self.page.timezone_city_combo.currentIndexChanged[int].connect(
+                self.cityChanged)
+        except Exception as e:
             self.debug('Could not create timezone page: %s', e)
             self.page = None
 
@@ -275,7 +281,8 @@ class PageKde(plugin.PluginUI):
         self.page.timezone_zone_combo.clear()
         for pair in shortlist:
             self.page.timezone_zone_combo.addItem(pair[0], pair[1])
-        self.page.timezone_zone_combo.insertSeparator(self.page.timezone_zone_combo.count())
+        self.page.timezone_zone_combo.insertSeparator(
+            self.page.timezone_zone_combo.count())
         for pair in longlist:
             self.page.timezone_zone_combo.addItem(pair[0], pair[2])
 
@@ -283,21 +290,25 @@ class PageKde(plugin.PluginUI):
     def populateCities(self, regionIndex):
         self.page.timezone_city_combo.clear()
 
-        code = str(self.page.timezone_zone_combo.itemData(regionIndex).toPyObject())
+        code = str(
+            self.page.timezone_zone_combo.itemData(regionIndex).toPyObject())
         countries = self.controller.dbfilter.get_countries_for_region(code)
-        if not countries: # must have been a country code itself
+        if not countries:  # must have been a country code itself
             countries = [code]
 
-        shortlist, longlist = self.controller.dbfilter.build_timezone_pairs(countries)
+        shortlist, longlist = self.controller.dbfilter.build_timezone_pairs(
+            countries)
 
         for pair in shortlist:
             self.page.timezone_city_combo.addItem(pair[0], pair[1])
         if shortlist:
-            self.page.timezone_city_combo.insertSeparator(self.page.timezone_city_combo.count())
+            self.page.timezone_city_combo.insertSeparator(
+                self.page.timezone_city_combo.count())
         for pair in longlist:
             self.page.timezone_city_combo.addItem(pair[0], pair[1])
 
-        return len(countries) == 1 and self.controller.dbfilter.get_default_for_region(countries[0])
+        return (len(countries) == 1 and
+                self.controller.dbfilter.get_default_for_region(countries[0]))
 
     # called when the region(zone) combo changes
     @plugin.only_this_page
@@ -306,10 +317,12 @@ class PageKde(plugin.PluginUI):
             return
 
         self.page.timezone_city_combo.blockSignals(True)
-        #self.page.timezone_city_combo.currentIndexChanged[int].disconnect(self.cityChanged)
+        # self.page.timezone_city_combo.currentIndexChanged[int].disconnect(
+        #    self.cityChanged)
         default = self.populateCities(regionIndex)
         self.page.timezone_city_combo.blockSignals(False)
-        #self.page.timezone_city_combo.currentIndexChanged[int].connect(self.cityChanged)
+        # self.page.timezone_city_combo.currentIndexChanged[int].connect(
+        #    self.cityChanged)
 
         if default:
             self.tzmap.set_timezone(default)
@@ -318,7 +331,8 @@ class PageKde(plugin.PluginUI):
 
     # called when the city combo changes
     def cityChanged(self, cityindex):
-        zone = str(self.page.timezone_city_combo.itemData(cityindex).toPyObject())
+        zone = str(
+            self.page.timezone_city_combo.itemData(cityindex).toPyObject())
         self.tzmap.zoneChanged.disconnect(self.mapZoneChanged)
         self.tzmap.set_timezone(zone)
         self.tzmap.zoneChanged.connect(self.mapZoneChanged)
@@ -331,7 +345,7 @@ class PageKde(plugin.PluginUI):
         for i in range(self.page.timezone_zone_combo.count()):
             code = str(self.page.timezone_zone_combo.itemData(i).toPyObject())
             countries = self.controller.dbfilter.get_countries_for_region(code)
-            if not countries: # must have been a country code itself
+            if not countries:  # must have been a country code itself
                 countries = [code]
             if loc.country in countries:
                 self.page.timezone_zone_combo.setCurrentIndex(i)
@@ -348,13 +362,14 @@ class PageKde(plugin.PluginUI):
         self.page.timezone_zone_combo.blockSignals(False)
         self.page.timezone_city_combo.blockSignals(False)
 
-    def set_timezone (self, timezone):
+    def set_timezone(self, timezone):
         self.refresh_timezones()
         self.tzmap.set_timezone(timezone)
 
-    def get_timezone (self):
+    def get_timezone(self):
         return self.tzmap.get_timezone()
 
+
 class PageDebconf(plugin.PluginUI):
     plugin_title = 'ubiquity/text/timezone_heading_label'
 
@@ -365,6 +380,7 @@ class PageDebconf(plugin.PluginUI):
     def plugin_set_online_state(self, state):
         self.online = state
 
+
 class PageNoninteractive(plugin.PluginUI):
     def __init__(self, controller, *args, **kwargs):
         self.controller = controller
@@ -381,6 +397,7 @@ class PageNoninteractive(plugin.PluginUI):
         """Get the current selected timezone."""
         return self.timezone
 
+
 class Page(plugin.Plugin):
     def prepare(self, unfiltered=False):
         # TODO: This can go away once we have the ability to abort wget/rdate
@@ -391,6 +408,14 @@ class Page(plugin.Plugin):
         clock_script = '/usr/share/ubiquity/clock-setup'
         env = {'PATH': '/usr/share/ubiquity:' + os.environ['PATH']}
 
+        # TODO: replace with more general version pushed down into
+        # is_automatic or similar
+        try:
+            self.automatic_page = (
+                self.db.get("ubiquity/automatic/timezone") == "true")
+        except debconf.DebconfError:
+            self.automatic_page = False
+
         if unfiltered:
             # In unfiltered mode, localechooser is responsible for selecting
             # the country, so there's no need to repeat the job here.
@@ -407,7 +432,10 @@ class Page(plugin.Plugin):
             self.collator = icu.Collator.createInstance(icu.Locale(locale))
         except:
             self.collator = None
-        if not 'UBIQUITY_AUTOMATIC' in os.environ:
+        if self.is_automatic or self.automatic_page:
+            if self.db.fget('time/zone', 'seen') == 'true':
+                self.set_di_country(self.db.get('time/zone'))
+        else:
             self.db.fset('time/zone', 'seen', 'false')
             cc = self.db.get('debian-installer/country')
             try:
@@ -418,7 +446,7 @@ class Page(plugin.Plugin):
                 pass
         self.preseed('tzsetup/selected', 'false')
         questions = ['^time/zone$', '^tzsetup/detected$', 'CAPB', 'PROGRESS']
-        return ([clock_script], questions, env)
+        return [clock_script], questions, env
 
     def capb(self, capabilities):
         self.frontend.debconf_progress_cancellable(
@@ -444,7 +472,16 @@ class Page(plugin.Plugin):
                     zone = choices_c[0]
             self.ui.set_timezone(zone)
 
-        return plugin.Plugin.run(self, priority, question)
+        if self.automatic_page:
+            # TODO: invade frontend's privacy to avoid entering infinite
+            # loop when trying to back up over timezone question (which
+            # isn't possible anyway since it's just after partitioning);
+            # this needs to be tidied up substantially when generalising
+            # ubiquity/automatic/*
+            self.frontend.backup = False
+            return True
+        else:
+            return plugin.Plugin.run(self, priority, question)
 
     def get_default_for_region(self, region):
         try:
@@ -461,10 +498,12 @@ class Page(plugin.Plugin):
         return s[0]
 
     def get_countries_for_region(self, region):
-        if region in self.regions: return self.regions[region]
+        if region in self.regions:
+            return self.regions[region]
 
         try:
-            codes = self.choices_untranslated('localechooser/countrylist/%s' % region)
+            codes = self.choices_untranslated(
+                'localechooser/countrylist/%s' % region)
         except debconf.DebconfError:
             codes = []
         self.regions[region] = codes
@@ -475,7 +514,8 @@ class Page(plugin.Plugin):
         total = []
         continents = self.choices_untranslated('localechooser/continentlist')
         for continent in continents:
-            country_codes = self.choices_untranslated('localechooser/countrylist/%s' % continent.replace(' ', '_'))
+            country_codes = self.choices_untranslated(
+                'localechooser/countrylist/%s' % continent.replace(' ', '_'))
             for c in country_codes:
                 shortlist = self.build_shortlist_timezone_pairs(c, sort=False)
                 longlist = self.build_longlist_timezone_pairs(c, sort=False)
@@ -492,26 +532,27 @@ class Page(plugin.Plugin):
     # Returns [('translated country name', None, 'region code')...] list
     def build_region_pairs(self):
         continents = self.choices_display_map('localechooser/continentlist')
-        names, codes = zip(*continents.items())
+        names, codes = list(zip(*continents.items()))
         codes = [c.replace(' ', '_') for c in codes]
 
         nones = [None for _ in continents]
-        pairs = zip(names, nones, codes)
+        pairs = list(zip(names, nones, codes))
         pairs.sort(key=self.collation_key)
         return pairs
 
     # Returns [('translated short list of countries', 'timezone')...] list
     def build_shortlist_region_pairs(self, language_code):
         try:
-            shortlist = self.choices_display_map('localechooser/shortlist/%s' % language_code)
+            shortlist = self.choices_display_map(
+                'localechooser/shortlist/%s' % language_code)
             # Remove any 'other' entry
             for pair in shortlist.items():
                 if pair[1] == 'other':
                     del shortlist[pair[0]]
                     break
-            names, codes = zip(*shortlist.items())
+            names, codes = list(zip(*shortlist.items()))
             nones = [None for _ in names]
-            shortlist = zip(names, codes, nones)
+            shortlist = list(zip(names, codes, nones))
             shortlist.sort(key=self.collation_key)
             return shortlist
         except debconf.DebconfError:
@@ -526,7 +567,8 @@ class Page(plugin.Plugin):
 
         longlist = []
         for country_code in country_codes:
-            longlist += self.build_longlist_timezone_pairs(country_code, sort=False)
+            longlist += self.build_longlist_timezone_pairs(
+                country_code, sort=False)
         longlist.sort(key=self.collation_key)
 
         # There may be duplicate entries in the shortlist and longlist.
@@ -548,12 +590,13 @@ class Page(plugin.Plugin):
 
     def build_shortlist_timezone_pairs(self, country_code, sort=True):
         try:
-            shortlist = self.choices_display_map('tzsetup/country/%s' % country_code)
-            for pair in shortlist.items():
+            shortlist = self.choices_display_map(
+                'tzsetup/country/%s' % country_code)
+            for pair in list(shortlist.items()):
                 # Remove any 'other' entry, we don't need it
                 if pair[1] == 'other':
                     del shortlist[pair[0]]
-            shortlist = shortlist.items()
+            shortlist = list(shortlist.items())
             if sort:
                 shortlist.sort(key=self.collation_key)
             return shortlist
@@ -563,14 +606,17 @@ class Page(plugin.Plugin):
     def get_country_name(self, country):
         # Relatively expensive algorithmically, but we don't call this often.
         try:
-            continents = self.choices_untranslated('localechooser/continentlist')
+            continents = self.choices_untranslated(
+                'localechooser/continentlist')
             for continent in continents:
-                choices = self.choices_display_map('localechooser/countrylist/%s' % continent.replace(' ', '_'))
+                choices = self.choices_display_map(
+                    'localechooser/countrylist/%s' %
+                    continent.replace(' ', '_'))
                 for name, code in choices.items():
                     if code == country:
                         return name
-        except debconf.DebconfError, e:
-            print "Couldn't get country name for %s: %s" % (country, e)
+        except debconf.DebconfError as e:
+            print("Couldn't get country name for %s: %s" % (country, e))
         return None
 
     def get_city_name_from_tzdata(self, tz):
@@ -585,8 +631,8 @@ class Page(plugin.Plugin):
                 for name, code in zones.items():
                     if code == city:
                         return name
-        except debconf.DebconfError, e:
-            print "Couldn't get city name for %s: %s" % (tz, e)
+        except debconf.DebconfError as e:
+            print("Couldn't get city name for %s: %s" % (tz, e))
         return None
 
     def get_fallback_translation_for_tz(self, country, tz):
@@ -600,7 +646,7 @@ class Page(plugin.Plugin):
             # First, try tzdata's translation.
             city_name = self.get_city_name_from_tzdata(tz)
             if city_name is None:
-                city_name = tz # fall back to ASCII name
+                city_name = tz  # fall back to ASCII name
             city_name = city_name.split('/')[-1]
             return "%s (%s)" % (country_name, city_name)
         else:
@@ -609,13 +655,13 @@ class Page(plugin.Plugin):
     # Returns [('translated long list of timezones', 'timezone')...] list
     def build_longlist_timezone_pairs(self, country_code, sort=True):
         if 'LANG' not in os.environ:
-            return [] # ?!
+            return []  # ?!
         locale = os.environ['LANG'].rsplit('.', 1)[0]
         tz_format = icu.SimpleDateFormat('VVVV', icu.Locale(locale))
-        now = time.time()*1000
+        now = time.time() * 1000
         rv = []
         try:
-            locs = self.tzdb.cc_to_locs[country_code] # BV failed?
+            locs = self.tzdb.cc_to_locs[country_code]  # BV failed?
         except:
             # Some countries in tzsetup don't exist in zone.tab...
             # Specifically BV (Bouvet Island) and
@@ -634,9 +680,10 @@ class Page(plugin.Plugin):
             # for the number part.  icu does not indicate a 'translation
             # failure' like this in any way...
             if (translated is None or
-                re.search('.*[-+][0-9][0-9]:?[0-9][0-9]$', translated)):
+                    re.search('.*[-+][0-9][0-9]:?[0-9][0-9]$', translated)):
                 # Wasn't something that icu understood...
-                name = self.get_fallback_translation_for_tz(country_code, location.zone)
+                name = self.get_fallback_translation_for_tz(
+                    country_code, location.zone)
                 rv.append((name, location.zone))
             else:
                 rv.append((translated, location.zone))
@@ -647,10 +694,11 @@ class Page(plugin.Plugin):
     # Returns [('translated long list of timezones', 'timezone')...] list
     def build_longlist_timezone_pairs_by_continent(self, continent):
         if 'LANG' not in os.environ:
-            return [] # ?
+            return []  # ?
         rv = []
         try:
-            regions = self.choices_untranslated('localechooser/countrylist/%s' % continent)
+            regions = self.choices_untranslated(
+                'localechooser/countrylist/%s' % continent)
             for region in regions:
                 rv += self.build_longlist_timezone_pairs(region, sort=False)
             rv.sort(key=self.collation_key)
@@ -658,22 +706,24 @@ class Page(plugin.Plugin):
             pass
         return rv
 
+    def set_di_country(self, zone):
+        location = self.tzdb.get_loc(zone)
+        if location:
+            self.preseed('debian-installer/country', location.country)
+
     def ok_handler(self):
         zone = self.ui.get_timezone()
         if zone is None:
             zone = self.db.get('time/zone')
         else:
             self.preseed('time/zone', zone)
-        for location in self.tzdb.locations:
-            if location.zone == zone:
-                self.preseed('debian-installer/country', location.country)
-                break
+        self.set_di_country(zone)
         plugin.Plugin.ok_handler(self)
 
     def cleanup(self):
         plugin.Plugin.cleanup(self)
-        self.ui.controller.set_locale(
-                i18n.reset_locale(self.frontend, just_country=True))
+        self.ui.controller.set_locale(i18n.reset_locale(self.frontend))
+
 
 class Install(plugin.InstallPlugin):
     def prepare(self, unfiltered=False):
@@ -687,4 +737,5 @@ class Install(plugin.InstallPlugin):
 
     def install(self, target, progress, *args, **kwargs):
         progress.info('ubiquity/install/timezone')
-        return plugin.InstallPlugin.install(self, target, progress, *args, **kwargs)
+        return plugin.InstallPlugin.install(
+            self, target, progress, *args, **kwargs)
